@@ -37,7 +37,6 @@ class Saori(SAORI):
         bus.add_signal_watch()
         bus.connect('message', self.on_message)
         self.filepath = None
-        self.state = 0 # 0/1/2 - stop/playing/paused
         self.loop = False
 
     def check_import(self):
@@ -58,12 +57,10 @@ class Saori(SAORI):
             elif argv[0] in ['play', 'loop']:
                 if argv[0] == 'loop':
                     self.loop = True
-                if self.state == 2:
-                    self.state = 1
+                if self.player.get_state(timeout=Gst.SECOND)[1] == Gst.State.PAUSED:
                     self.player.set_state(Gst.State.PLAYING)
                     return self.RESPONSE[204]
-                elif self.state == 1:
-                    self.state = 2
+                elif self.player.get_state(timeout=Gst.SECOND)[1] == Gst.State.PLAYING:
                     self.player.set_state(Gst.State.PAUSED)
                     return self.RESPONSE[204]
                 if self.filepath is not None and os.path.isfile(self.filepath):
@@ -85,12 +82,8 @@ class Saori(SAORI):
             self.player.set_state(Gst.State.NULL)
             if self.loop:
                 self.player.set_state(Gst.State.PLAYING)
-                self.state = 1
-            else:
-                self.state = 0
         elif t == Gst.MessageType.ERROR:
             self.player.set_state(Gst.State.NULL)
             err, debug = message.parse_error()
             logging.error('Error: {0}, {1}'.format(err, debug))
-            self.state = 0
             self.loop = False
