@@ -140,11 +140,9 @@ class TransparentWindow(Gtk.Window):
         self.__child.queue_draw()
 
     def update_shape(self, surface):
-        try:
-            region = Gdk.cairo_region_create_from_surface(surface)
-            self.input_shape_combine_region(region)
-        except: # Ask pycairo upstream to support cairo_region_t
-            pass
+        region = Gdk.cairo_region_create_from_surface(surface)
+        self.get_window().input_shape_combine_region(
+            region, self.offset['left'], self.offset['top'])
 
 def get_png_size(path):
     if not path or not os.path.exists(path):
