@@ -77,6 +77,7 @@ class Sakura:
     BROWSE_MODE = 1
     SELECT_MODE = 2
     PAUSE_MODE  = 3
+    WAIT_MODE   = 4 
     # script origins
     FROM_SSTP_CLIENT = 1
     FROM_GHOST       = 2
@@ -133,8 +134,12 @@ class Sakura:
             self.audio_player = Gst.ElementFactory.make('playbin', 'player')
             fakesink = Gst.ElementFactory.make('fakesink', 'fakesink')
             self.audio_player.set_property('video-sink', fakesink)
+            bus = self.audio_player.get_bus()
+            bus.add_signal_watch()
+            bus.connect('message', self.on_audio_message)
         else:
             self.audio_player = None
+        self.audio_loop = False
 
     def set_responsible(self, request_method):
         self.request_parent = request_method
@@ -318,6 +323,25 @@ class Sakura:
     def leave_temp_mode(self):
         self.__temp_mode = 0
 
+    def on_audio_message(self, bus, message):
+        if message is None: # XXX: workaround for Gst Version < 0.11
+            if self.script_mode == self.WAIT_MODE:
+                self.script_mode = self.BROWSE_MODE
+            return
+        t = message.type
+        if t == Gst.MessageType.EOS:
+            self.audio_player.set_state(Gst.State.NULL)
+            if self.script_mode == self.WAIT_MODE:
+                assert not self.audio_loop
+                self.script_mode = self.BROWSE_MODE
+            if self.audio_loop:
+                self.audio_player.set_state(Gst.State.PLAYING)
+        elif t == Gst.MessageType.ERROR:
+            self.audio_player.set_state(Gst.State.NULL)
+            err, debug = message.parse_error()
+            logging.error('Error: {0}, {1}'.format(err, debug))
+            self.audio_loop = False
+
     def set_surface(self, desc, alias, surface, name, surface_dir, tooltips):
         self.surface.new(desc, alias, surface, name, surface_dir, tooltips)
         for side in range(2, self.char):
@@ -1042,9 +1066,6 @@ class Sakura:
         self.balloon.hide_sstp_message()
         if reset_surface:
             self.set_surface_default()
-            self.notify_event('OnSurfaceChange',
-                              self.get_surface_id(0),
-                              self.get_surface_id(1))
             self.balloon.set_balloon_default()
         elif self.get_surface_id(0) != '0' or \
              self.get_surface_id(1) != '10':
@@ -1128,6 +1149,7 @@ class Sakura:
         self.balloon.finalize()
         if self.audio_player is not None:
             self.audio_player.set_state(Gst.State.NULL)
+        self.audio_loop = False
 
     def process_script(self):
         now = time.time()
@@ -1158,6 +1180,8 @@ class Sakura:
             ##if idle > self.PAUSE_TIMEOUT:
             ##    self.script_mode = self.BROWSE_MODE
             pass
+        elif self.script_mode == self.WAIT_MODE:
+            pass
         elif self.processed_script or self.processed_text:
             self.interpret_script()
         elif self.script_post_proc:
@@ -1391,10 +1415,6 @@ class Sakura:
         else:
             self.set_surface_id(self.script_side, surface_id)
             self.surface.show(self.script_side)
-            ## FIXME: self.script_side > 1
-            self.notify_event('OnSurfaceChange',
-                              self.get_surface_id(0),
-                              self.get_surface_id(1))
         if self.script_side in [0, 1] and not self.__boot[self.script_side]:
             self.__boot[self.script_side] = 1
 
@@ -1577,8 +1597,32 @@ class Sakura:
             self.audio_player.set_state(Gst.State.NULL)
             self.audio_player.set_property(
                 'uri', 'file://' + urllib.parse.quote(path))
+            self.audio_loop = False
             self.audio_player.set_state(Gst.State.PLAYING)
 
+    def __yen_8(self, args):
+        if self.audio_player is None:
+            return
+        filename = self.expand_meta(args[0])
+        filename = get_normalized_path(filename)
+        basename, ext = os.path.splitext(filename)
+        ext = ext.lower()
+        if ext != b'.wav':
+            return
+        path = os.path.join(self.get_prefix(), b'ghost/master', filename)
+        if os.path.isfile(path):
+            self.audio_player.set_state(Gst.State.NULL)
+            self.audio_player.set_property(
+                'uri', 'file://' + urllib.parse.quote(path))
+            self.audio_loop = False
+            self.audio_player.set_state(Gst.State.PLAYING)
+
+    def __yen__V(self, args):
+        if self.audio_loop:
+            return # nothing to do
+        if self.audio_player.get_state(timeout=Gst.SECOND)[1] == Gst.State.PLAYING:
+            self.script_mode = self.WAIT_MODE
+
     def __yen_exclamation(self, args): ## FIXME
         if not args:
             return
@@ -1696,6 +1740,58 @@ class Sakura:
                     gsettings.set_string('picture-options', opt)
                 else:
                     pass # not implemented yet
+        elif args[0] == 'sound':
+            command = args[1]
+            if self.audio_player is None:
+                return
+            if command == 'stop':
+                self.audio_player.set_state(Gst.State.NULL)
+                self.audio_loop = False
+            elif command == 'play':
+                filename = args[2]
+                filename = get_normalized_path(filename)
+                path = os.path.join(self.get_prefix(),
+                                    b'ghost/master', filename)
+                if os.path.isfile(path):
+                    self.audio_player.set_state(Gst.State.NULL)
+                    self.audio_player.set_property(
+                        'uri', 'file://' + urllib.parse.quote(path))
+                    self.audio_loop = False
+                    self.audio_player.set_state(Gst.State.PLAYING)
+            elif command == 'cdplay':
+                self.audio_player.set_state(Gst.State.NULL)
+                try:
+                    track = int(args[2])
+                except:
+                    return
+                self.audio_player.set_property(
+                    'uri', 'cdda://{}'.format(track))
+                self.audio_loop = False
+                self.audio_player.set_state(Gst.State.PLAYING)
+            elif command == 'loop':
+                filename = args[2]
+                filename = get_normalized_path(filename)
+                path = os.path.join(self.get_prefix(),
+                                    b'ghost/master', filename)
+                if os.path.isfile(path):
+                    self.audio_player.set_state(Gst.State.NULL)
+                    self.audio_player.set_property(
+                        'uri', 'file://' + urllib.parse.quote(path))
+                    self.audio_loop = True
+                    self.audio_player.set_state(Gst.State.PLAYING)
+            elif command == 'wait':
+                if self.audio_loop:
+                    return # nothing to do
+                if self.audio_player.get_state(timeout=Gst.SECOND)[1] == Gst.State.PLAYING:
+                    self.script_mode = self.WAIT_MODE
+            elif command == 'pause':
+                if self.audio_player.get_state(timeout=Gst.SECOND)[1] == Gst.State.PLAYING:
+                    self.audio_player.set_state(Gst.State.PAUSED)
+            elif command == 'resume':
+                if self.audio_player.get_state(timeout=Gst.SECOND)[1] == Gst.State.PAUSED:
+                    self.audio_player.set_state(Gst.State.PLAYING)
+            else:
+                pass ## FIXME
         elif args[0] == '*':
             self.balloon.append_sstp_marker(self.script_side)
         else:
@@ -1729,6 +1825,8 @@ class Sakura:
 
     __script_tag = {
         r'\e': __yen_e,
+        r'\y': __yen_e,
+        r'\z': __yen_e,
         r'\0': __yen_0,
         r'\h': __yen_0,
         r'\1': __yen_1,
@@ -1762,6 +1860,8 @@ class Sakura:
         r'\_m': __yen__m,
         r'\_u': __yen__u,
         r'\_v': __yen__v,
+        r'\8': __yen_8,
+        r'\_V': __yen__V,
         r'\!': __yen_exclamation,
         r'\__c': __yen___c,
         r'\__t': __yen___t, 
@@ -1862,6 +1962,9 @@ class Sakura:
                 buf.append(str(scrn_h))
             elif chunk[1] == '%et':
                 buf.append('{0:d}万年'.format(self.current_time[7]))
+            elif chunk[1] == '%wronghour':
+                wrongtime = time.time() + random.choice((-2, -1, 1, 2)) * 3600
+                buf.append(str(time.localtime(wrongtime)[3]))
             elif chunk[1] == '%exh':
                 buf.append(str(self.get_uptime()))
             elif chunk[1] in ['%ms', '%mz', '%ml', '%mc', '%mh', \
@@ -1901,12 +2004,16 @@ class Sakura:
             pass
         self.sstp_handle = None
 
-    def close(self):
+    def close(self, reason='user'):
         if self.busy():
-            Gdk.beep() ## FIXME
-            return
+            if reason == 'user':
+                Gdk.beep() ## FIXME
+                return
+            else: # shutdown
+                if self.updateman.is_active():
+                    self.updateman.interrupt()
         self.reset_script(1)
-        self.enqueue_event('OnClose')
+        self.enqueue_event('OnClose', reason)
 
     def about(self):
         if self.busy():
