@@ -39,6 +39,7 @@ class Surface:
         self.request_parent = lambda *a: None # dummy
         self.mikire = 0
         self.kasanari = 0
+        self.key_press_count = 0
 
     def set_responsible(self, request_method):
         self.request_parent = request_method
@@ -72,8 +73,10 @@ class Surface:
             window.set_skip_taskbar_hint(True)
         window.connect('delete_event', self.delete)
         window.connect('key_press_event', self.key_press)
+        window.connect('key_release_event', self.key_press)
         window.connect('window_state_event', self.window_state)
-        window.set_events(Gdk.EventMask.KEY_PRESS_MASK)
+        window.set_events(Gdk.EventMask.KEY_PRESS_MASK|
+                          Gdk.EventMask.KEY_RELEASE_MASK)
         window.realize()
         return window
 
@@ -128,13 +131,21 @@ class Surface:
     def key_press(self, window, event):
         name = self.keymap_old.get(event.keyval, event.string)
         keycode = self.keymap_new.get(event.keyval, event.string)
-        if event.get_state() & (Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK):
+        if event.type == Gdk.EventType.KEY_RELEASE:
+            self.key_press_count = 0
+            return True
+        if not (event.type == Gdk.EventType.KEY_PRESS):
+            return False
+        self.key_press_count += 1
+        if event.get_state() & \
+                (Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK):
             if name == 'f12':
                 logging.info('reset surface position')
                 self.reset_position()
         if name or keycode:
             self.request_parent(
-                'NOTIFY', 'notify_event', 'OnKeyPress', name, keycode)
+                'NOTIFY', 'notify_event', 'OnKeyPress', name, keycode,
+                self.key_press_count)
         return True
 
     def window_stick(self, stick):
@@ -683,6 +694,9 @@ class SurfaceWindow:
         self.darea.connect('motion_notify_event', self.motion_notify)
         self.darea.connect('drag_data_received', self.drag_data_received)
         self.darea.connect('scroll_event', self.scroll)
+        if self.side == 0:
+            screen = self.window.get_screen()
+            screen.connect('size-changed', self.display_changed)
         # DnD data types
         dnd_targets = [Gtk.TargetEntry.new('text/uri-list', 0, 0)]
         self.darea.drag_dest_set(Gtk.DestDefaults.ALL, dnd_targets,
@@ -692,6 +706,12 @@ class SurfaceWindow:
         self.window.realize()
         self.window.get_window().set_background_pattern(None) # XXX
 
+    def display_changed(self, screen):
+        left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
+        self.request_parent(
+            'NOTIFY', 'notify_event', 'OnDisplayChange',
+            Gdk.Visual.get_best_depth(), scrn_w, scrn_h)
+
     def set_responsible(self, request_method):
         self.request_parent = request_method
 
@@ -793,6 +813,12 @@ class SurfaceWindow:
             self.set_position(x, y)
         if self.side < 2:
             self.request_parent('NOTIFY', 'notify_observer', 'set surface')
+        self.request_parent(
+            'NOTIFY', 'notify_event',
+            'OnSurfaceChange',
+            self.request_parent('GET', 'get_surface_id', 0),
+            self.request_parent('GET', 'get_surface_id', 1),
+            "{},{},{},{}".format(self.side, self.surface_id, w, h))
 
     def iter_mayuna(self, surface_width, surface_height, mayuna, done):
         for surface_id, interval, method, args in mayuna.patterns:
@@ -1116,6 +1142,14 @@ class SurfaceWindow:
         if self.dragged:
             self.dragged = False
             self.set_alignment_current()
+            x = int(event.x)
+            y = int(event.y)
+            scale = self.scale
+            x = int(x * 100 / scale)
+            y = int(y * 100 / scale)
+            self.request_parent(
+                'NOTIFY', 'notify_event',
+                'OnMouseDragEnd', x, y, '', self.side, self.__current_part, '')
         self.x_root = None
         self.y_root = None
         return True
@@ -1153,6 +1187,11 @@ class SurfaceWindow:
             if state & Gdk.ModifierType.BUTTON1_MASK:
                 if self.x_root is not None and \
                    self.y_root is not None:
+                    if not self.dragged:
+                        self.request_parent(
+                            'NOTIFY', 'notify_event',
+                            'OnMouseDragStart', x, y, '',
+                            self.side, self.__current_part, '')
                     self.dragged = True
                     x_delta = int(event.x_root - self.x_root)
                     y_delta = int(event.y_root - self.y_root)
@@ -1190,6 +1229,10 @@ class SurfaceWindow:
         if bind_id in self.bind:
             current = self.bind[bind_id][1]
             self.bind[bind_id][1] = not current
+            self.request_parent('NOTIFY', 'notify_event',
+                                'OnDressupChanged', self.side,
+                                self.bind[bind_id][0],
+                                1 if self.bind[bind_id][1] else 0)
             self.reset_surface()
 
     def window_enter_notify(self, window, event):
