@@ -675,6 +675,10 @@ class Application:
         return self.__menu_owner.get_current_balloon_directory()
 
     def start_sakura_cb(self, key): ## FIXME
+        sakura_name = self.ghosts[key].instance.get_selfname(default='')
+        name = self.ghosts[key].instance.get_name(default='')
+        self.__menu_owner.notify_event(
+            'OnGhostCalling', sakura_name, 'manual', name, key)
         self.start_sakura(key, init=1) # XXX
 
     def select_sakura(self, key):
@@ -865,7 +869,17 @@ class Application:
                 pass
         return None
 
+    def exit_handler(self, sig_no, frame=None):
+        self.close_all_ghosts(reason='shutdown')
+
     def run(self):
+        if os.name == "nt":
+            # The SIGTERM signal is not generated under Windows NT.
+            import win32api
+            win32api.SetConsoleCtrlHandler(self.exit_handler, True)
+        else:
+            import signal
+            signal.signal(signal.SIGTERM, self.exit_handler)
         self.timeout_id = GLib.timeout_add(100, self.do_idle_tasks) # 100[ms]
         Gtk.main()
 
@@ -931,9 +945,9 @@ class Application:
         elif self.current_sakura == sakura.key:
             self.select_current_sakura()
 
-    def close_all_ghosts(self):
+    def close_all_ghosts(self, reason='user'):
         for sakura in self.get_working_ghost():
-            sakura.close()
+            sakura.close(reason)
 
     def quit(self):
         GLib.source_remove(self.timeout_id)
@@ -981,8 +995,10 @@ class Application:
         elif not event:
             proc()
         else:
-            name = self.ghosts[key].instance.get_selfname(default='')
-            sakura.enqueue_event('OnGhostChanging', name, method, proc=proc)
+            sakura_name = self.ghosts[key].instance.get_selfname(default='')
+            name = self.ghosts[key].instance.get_name(default='')
+            sakura.enqueue_event(
+                'OnGhostChanging', sakura_name, method, name, key, proc=proc)
 
     def stop_sakura(self, sakura, starter=None, *args):
         sakura.finalize()
